//*****************************************
// AT mega328P-AU(32pin TQFP) Six Nixie Clock Program
// CPU Clock:8MHz
//   iq[Yf[^@WȂBIV[^(8MHz)ɂj
//
// File Name:nixie.c
//
//  2016/09/16 Taishi
//*****************************************

#include	<avr/pgmspace.h>
#include	<avr/io.h>

#include	"six_nixie.h"
#include	"nixie.h"

// **********************************************************
// *********************** NIXE TUBE ************************
// **********************************************************
//

// **********************************************************
//@jLV[ǂ͂O`Xƍ_\
//@@jLV[ǑI
//@@@ŏʁ@obOfgf
//@@@Qʁ@obPfgf
//@@@Rʁ@obQfgf
//@@@Sʁ@obSfgf
//@@@Tʁ@obSfgf
//@@@ŉʁ@obTfgf
//
//@@\locSCTCUCV̂Srbĝabcōs
//@@@V@U@T@S
//@@@k@k@k@k@||@O
//@@@k@k@k@g@||@P
//@@@k@k@g@k@||@Q
//@@@k@k@g@g@||@R
//@@@k@g@k@k@||@S
//@@@k@g@k@g@||@T
//@@@k@g@g@k@||@U
//@@@k@g@g@g@||@V
//@@@g@k@k@k@||@W
//@@@g@k@k@g@||@X
//@@@g@k@g@k@||@
//@@@g@k@g@g@||@
//@@@g@g@k@k@||@
//@@@g@g@k@g@||@
//@@@g@g@g@k@||@
//@@@g@g@g@g@||@
//
//@@_\iRj
//@@@ocPFkœ_Agŏ
//
// **********************************************************

unsigned char	path_limit = 1;

static	unsigned char	nixie_data[KETA_MAX+1];
static	unsigned char	nixie_dot[KETA_MAX+1];
static	unsigned char	nixie_select[KETA_MAX+1];
static	unsigned char	nixie_act_num = 0;
static	unsigned char	dynamic_nixie = 0;

#define		NIXE_DATA_MASK		0x0d	// PD 1,4--7 Data Mask
#define		NIXE_SELECT_CLR		0xc0	// PC0 -- 5 Clear
#define		NIXE_NUMBER_OFF		0xf0	// PD4--7:'H'
#define		NIXE_DOT_ON			0xfd	// PD1:'L' 
#define		NIXE_DOT_OFF		0x02	// PD1:'H' 
#define		NIXE_DOT		0x02	// PD1 
//******************************
//  Nixe\@
// *****************************
void	init_nixe(void)
{
	unsigned char	uc;

	PORTC &= NIXE_SELECT_CLR;					// IȂ
	PORTD &= NIXE_DATA_MASK;					// A_}XN
	PORTD |= NIXE_NUMBER_OFF;					// Ȃ
	PORTD |= NIXE_DOT_OFF;						// _OFF
	DDRD |= ((1 << DDD1) | (1 << DDD4) | (1 << DDD5) | (1 << DDD6) | (1 << DDD7));					// Port-D Bit1,4567 : Output
	DDRC |= ((1 << DDC0) | (1 << DDC1) | (1 << DDC2) | (1 << DDC3) | (1 << DDC4) | (1 << DDC5));	// Port-C Bit012345: Output

// \A_ f[^
	for(uc = 0;uc <= KETA_MAX; uc++){
		nixie_dot[uc] = NIXE_DOT_OFF;			// _NA
		nixie_data[uc] = NIXE_NUMBER_OFF;		// NA
	}

// I f[^
	nixie_select[0] = 0x01;				// ŏ
	nixie_select[1] = 0x02;				// Q
	nixie_select[2] = 0x04;				// R
	nixie_select[3] = 0x08;				// S
	nixie_select[4] = 0x10;				// T
	nixie_select[5] = 0x20;				// ŉ


	nixie_act_num = 0;

	dynamic_nixie = 0;

}
//******************************
// jLV[Ǖ\
// sQ̊荞݂ŌĂ΂
//  d͖TO`Ȃ邪A邭Ȃ
//******************************
void	proc_dynamic_nixie(void)
{
	static	unsigned char	path=0;

	if(dynamic_nixie == 1){
		PORTC &= NIXE_SELECT_CLR;						// [0xc0]IȂ
		PORTD &= NIXE_DATA_MASK;						// [0x0d] OA_nee
		if(path == 0){
			if(nixie_act_num <= (g_keta - 1)){
				PORTD |= nixie_data[nixie_act_num];				// 
				PORTC |= nixie_select[nixie_act_num];			// I
			}else if(nixie_act_num == (g_keta - 1)+1){
				PORTD |= NIXE_NUMBER_OFF;						// 
				if(nixie_dot[1] == NIXE_DOT_OFF){				// [0x02]_nee̎
					PORTD &= ~NIXE_DOT;							// _nee
				}else{
					PORTD |= NIXE_DOT;							// _nm
				}
				PORTC |= nixie_select[1];						// I
			}else if(nixie_act_num == (g_keta - 1)+2){
				PORTD |= NIXE_NUMBER_OFF;						// 
				if(nixie_dot[3] == NIXE_DOT_OFF){				// [0x02]_nee̎
					PORTD &= ~NIXE_DOT;							// _nee
				}else{
					PORTD |= NIXE_DOT;							// _nm
				}
				PORTC |= nixie_select[3];						// I
			}
			if((g_keta - 1)+2 <= nixie_act_num){
				nixie_act_num = 0;
			}else{
				nixie_act_num++;
			}
		}else{
			PORTD |= NIXE_NUMBER_OFF;						// 
		}

		path++;
		if(path_limit <= path){
			path = 0;
		}

	}
}

//******************************
// Dynamic Nixie display Active/Non active
//  on : 1= Active, other=Non Active
// *****************************
void	dynamic_nixie_active(unsigned char on)
{
	dynamic_nixie = on;
}
//******************************
// Nixie _\
//       k : ʒuiO`TAOFŏʁEEETFŉʁj
//  on_off : 1=ON, 0=OFF
// *****************************
void	disp_dynamic_nixie_dot(unsigned char k, unsigned char on)
{
	if(KETA_MAX < k){							// IlُȎ
		k = 0;									// Iɍŏʂɂ
	}
	if(on == 1){
		nixie_dot[k] = NIXE_DOT_ON;
	}else{
		nixie_dot[k] = NIXE_DOT_OFF;
	}
}
//******************************
// Nixie \
//  k : ʒuiO`TAOFŏʁEEETFŉʁj
//  n : O`XAPTiPT͋󔒁j
// *****************************
void	disp_dynamic_nixie_number(unsigned char k, unsigned char n)
{
	if(KETA_MAX < k){							// IlُȎ
		k = 0;									// Iɍŏʂɂ
	}
	nixie_data[k] = ((n << 4) & 0xf0);
}
//******************************
// R\
//       d : \iO`RAOF\ȂAPF̂݁AQF㑤̂݁ARFj
// *****************************
void	disp_dynamic_colon(unsigned char d)
{
	if(d == 3){
		nixie_dot[1] = NIXE_DOT_ON;
		nixie_dot[3] = NIXE_DOT_ON;
	}else if(d == 2){
		nixie_dot[1] = NIXE_DOT_ON;
		nixie_dot[3] = NIXE_DOT_OFF;
	}else if(d == 1){
		nixie_dot[1] = NIXE_DOT_OFF;
		nixie_dot[3] = NIXE_DOT_ON;
	}else{
		nixie_dot[1] = NIXE_DOT_OFF;
		nixie_dot[3] = NIXE_DOT_OFF;
	}
}

//******************************
// Nixe _\iX^eBbN\j
//       k : ʒuiO`TAOFŏʁEEETFŉʁj
//  on_off : NIXE_DOT_ON or NIXE_DOT_OFF
//  ܂ő̌ŕ\Ắ@̕\i_j
// *****************************
void	disp_static_nixie_dot(unsigned char k)
{
	if(KETA_MAX < k){							// IlُȎ
		k = 0;									// Iɍŏʂɂ
	}
	PORTC &= NIXE_SELECT_CLR;					// IȂ
	PORTD |= NIXE_NUMBER_OFF;					// Ȃ

	PORTD |= NIXE_DOT;							// _nm
	PORTC |= nixie_select[k];					// I

}
//******************************
// Nixie \iX^eBbN\j
//  k : ʒuiO`TAOFŏʁEEETFŉʁj
//  n : O`XAPTiPT͋󔒁j
//  ܂ő̌ŕ\Ắ@̕\i_j
// *****************************
void	disp_static_nixie_number(unsigned char k, unsigned char n)
{
	if(KETA_MAX < k){							// IlُȎ
		k = 0;									// Iɍŏʂɂ
	}
	PORTC &= NIXE_SELECT_CLR;					// IȂ
	PORTD &= NIXE_DATA_MASK;					// Ȃ
	PORTD &= ~NIXE_DOT;							// _Ȃ

	PORTD |= (n << 4);							// 
	PORTC |= nixie_select[k];					// I
}
